/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconfinder;

import samplemon.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 * A GUI for the BeaconFinderApp.
 *
 * @author Allen Miu
 */
public class GUI extends JTable
{
    protected JFrame f;

    private static String FONTNAME = "Arial";
    private static int FONTSTYLE = Font.BOLD;
    private static int FONTSIZE = 20;

    private JLabel xLabel, yLabel, zLabel;

    private TableModel model;
    static Font font;

    private CustomTableCellRenderer renderer;

    public class CustomTableCellRenderer extends DefaultTableCellRenderer
    {
	private Font font = new Font(FONTNAME, Font.PLAIN, FONTSIZE);

	public Component getTableCellRendererComponent
	    (JTable table, Object value, boolean isSelected,
	     boolean hasFocus, int row, int column)
	{
	    Component cell = super.getTableCellRendererComponent
		(table, value, isSelected, hasFocus, row, column);
	    
	    cell.setFont(font);
	    return cell;
	}
	
	public int getFontHeight()
	{
	    return FONTSIZE; // an approximation
	}
    }

    public TableCellRenderer getCellRenderer(int row, int col) {
	if(renderer == null) 
	    return super.getCellRenderer(row, col);

	if(row == model.curBeaconInd)
	    renderer.setBackground(Color.yellow);
	else
	    renderer.setBackground(Color.white);
	
	return renderer;
    }

    //private int nameWidth = 110;
    private int nameWidth = 180;
    //    private int statWidth = 57;
    private int statWidth = 115;

    //    public GUI(String title, BeaconFinderApp a, TableSorter s, int x, int y) 
    public GUI(String title, TableModel a, BarGraph b, SampleMonApp m, int x, int y) 
    {
	super(a);
	model = a;
	//renderer = new DefaultTableCellRenderer();
	renderer = new CustomTableCellRenderer();

	font = new Font(FONTNAME, FONTSTYLE, FONTSIZE-2);
	renderer.setFont(font);
	setRowHeight(renderer.getFontHeight()+2);
	//DefaultTableCellRenderer r = new DefaultTableCellRenderer();
	//r.setFont(font);
	//setDefaultRenderer(String.class, r);

	TableColumn tc = getColumn(model.columnNames[0]);
	tc.setMinWidth(nameWidth);
	tc.setMaxWidth(nameWidth);
	for(int i = 1; i < model.columnNames.length; i++) {
	    tc = getColumn(model.columnNames[i]);
	    tc.setMinWidth(statWidth);
	    tc.setMaxWidth(statWidth);
	}
	//tc = getColumn(model.columnNames[model.columnNames.length-1]);
	//tc.setMinWidth(120);
	//tc.setMaxWidth(120);


	getTableHeader().setFont(font);

	//setVisibleRowCount(4);
	setSelectionForeground(Color.white);
	setSelectionBackground(Color.red);
	setShowGrid(false);
	setAutoResizeMode(AUTO_RESIZE_OFF);
	setShowVerticalLines(true);
	setShowHorizontalLines(true);
	setCellSelectionEnabled(false);
	setColumnSelectionAllowed(false);
	setRowSelectionAllowed(false);

	/*
	  Coord dumb = new Coord(123, 123, "0123456789");
	  setPrototypeCellValue(dumb.toString());
	*/
	f = new CustomFrame(title);
	Container container = f.getContentPane();
	container.setLayout(new BorderLayout());
	if(b == null)
	    createCoordPanel(container);
	else
	    container.add(b, "Center");

	JScrollPane scroll = new JScrollPane(this);
	scroll.setBorder(createBorder());
	scroll.setPreferredSize(new Dimension(x, y));
	container.add(scroll, "North");
	scroll.setEnabled(false);	// for ipaq

	if(m != null)
	    container.add(m.getPanel(), "South");

	f.setBackground(Color.white);
	f.validate();
	f.pack();
	f.setVisible(true);
	setEnabled(false);

	Util.log("view created and exposed");
    }

    private Border createBorder() {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }

    private void createCoordPanel(Container cont) 
    {
	JPanel panel = new JPanel();
	panel.setLayout(new FlowLayout());
	xLabel = addInfoLabel(panel, " x:", "0");
	yLabel = addInfoLabel(panel, " y:", "0");
	zLabel = addInfoLabel(panel, " z:", "0");
	cont.add(panel, "South");
    }

    private JLabel addInfoLabel(JPanel p, String title, String value) 
    {
	JLabel t = new JLabel(title, JLabel.LEFT);
	p.add(t);
	
	if( value != null ) {
	    JLabel v = new JLabel(value, JLabel.RIGHT);
	    p.add(v);
	    return v;
	}
	return null;
    }

    public void setX(String s) {
	if(xLabel != null)
	    xLabel.setText(s);
    }

    public void setY(String s) {
	if(yLabel != null)
	    yLabel.setText(s);
    }

    public void setZ(String s) {
	if(zLabel != null)
	    zLabel.setText(s);
    }
}



