/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.data;

import cricketdaemon.reader.CartesianCoord;


/**
 * X, Y, Z coordinate.
 *
 *
 * @author Allen Miu
 */
public class Position
{
    public double x, y, z;
    
    public Position(double x, double y, double z)
    {
       this.x = x; this.y = y; this.z = z;
    }

    public Position copy()
    {
	return new Position(x, y, z);
    }
    
    public String toString()
    {
	if(invalid())
	    return "invalid";
	else
	    return "( "+x+" "+y+" "+z+" )";
    }

    public boolean equals(Object o)
    {
	if(!(o instanceof Position))
	    return false;
	Position p = (Position) o;

	return (p.x == x) && (p.y == y) && (p.z == z);
    }

    public static double sqr(double v) {
	return v*v;
    }

    public double dist(Position o)
    {
	return dist(o.x, o.y, o.z);
    }

    public double dist(double ox, double oy, double oz)
    {
	return Math.sqrt( sqr(x-ox) + sqr(y-oy) + sqr(z-oz) );
    }

    public boolean invalid()
    {
	return (Double.isNaN(x) || Double.isNaN(y) || Double.isNaN(z));
    }

    public static Position weightSum(double w1, Position p1, double w2, Position p2) 
    {
	if(p1 == null)
	    if(p2 == null)
		return null;
	    else
		return new Position(p2.x, p2.y, p2.z);

	if(p2 == null)
	    return new Position(p1.x, p1.y, p1.z);

	double x = w1*p1.x + w2*p2.x;
	double y = w1*p1.y + w2*p2.y;
	double z = w1*p1.z + w2*p2.z;

	return new Position(x, y, z);
    }
}
